unit main;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls, Synaser;

type

  { TfmMain }

  TfmMain = class(TForm)
    btOpen: TButton;
    btSendPDU: TButton;
    btRcv: TButton;
    btInit: TButton;
    btReadMsgs: TButton;
    btExec: TButton;
    btSendTextSMS: TButton;
    btClose: TButton;
    edCommand: TEdit;
    edLinuxPort: TEdit;
    edWindowsPort: TEdit;
    edMsgCenter: TEdit;
    edPhone: TEdit;
    edMsg: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Memo1: TMemo;
    RadioButton1: TRadioButton;
    rbLinux: TRadioButton;
    procedure btCloseClick(Sender: TObject);
    procedure btSendPDUClick(Sender: TObject);
    procedure btOpenClick(Sender: TObject);
    procedure btRcvClick(Sender: TObject);
    procedure btInitClick(Sender: TObject);
    procedure btReadMsgsClick(Sender: TObject);
    procedure btExecClick(Sender: TObject);
    procedure btSendTextSMSClick(Sender: TObject);
    procedure edMsgEnter(Sender: TObject);
    procedure edPhoneEnter(Sender: TObject);
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
  private
    function GetMsgNum(ALine: string): Integer;
    function SendPDU(MobileNumber, Msg: string): Boolean;
    { private declarations }
  public
    ser: TBlockSerial;
    PDUMsgNumber: Integer;

    function SendAndWait(AText: string; Terminator: char = #13; WaitingSeconds: Integer = 3): boolean;

    { public declarations }
  end; 

var
  fmMain: TfmMain;

implementation

{$R *.lfm}

{ TfmMain }

uses PDUFormat;


procedure Delay(ms: Cardinal);
var
  Start: Cardinal;
begin
  Start:= DateTimeToFileDate(Now);

  while ms + Start > DateTimeToFileDate(Now) do
    Application.ProcessMessages;
end;

procedure TfmMain.btOpenClick(Sender: TObject);
var
  i: Integer;
begin
    try
      Memo1.Lines.Add('Opening port');
      if rbLinux.Checked then
        ser.Connect(edLinuxPort.Text)
      else
        ser.Connect(edWindowsPort.Text);

      ser.Config(9600, 8, 'M', 0, True, True);
      ser.RaiseExcept:= false;
      Memo1.Lines.Add(ser.LastErrorDesc);
      SendAndWait('AT');

      btOpen.Enabled:= False;
      btExec.Enabled:= True;
      btInit.Enabled:= True;
      btReadMsgs.Enabled:= True;
      btSendPDU.Enabled:= True;
      btSendTextSMS.Enabled:= True;
      btRcv.Enabled:= True;
      btClose.Enabled:= True;

    except
      on e: exception do
      Memo1.Lines.Add(e.Message);
    end;
end;

procedure TfmMain.btSendPDUClick(Sender: TObject);
begin
  SendPDU(edPhone.Text, edMsg.Text);
end;

procedure TfmMain.btCloseClick(Sender: TObject);
begin
  ser.CloseSocket;
  btOpen.Enabled:= True;
  btExec.Enabled:= False;
  btInit.Enabled:= False;
  btReadMsgs.Enabled:= False;
  btSendPDU.Enabled:= False;
  btSendTextSMS.Enabled:= False;
  btRcv.Enabled:= False;
  btClose.Enabled:= False;

end;


procedure TfmMain.btRcvClick(Sender: TObject);
var
  DataR: Boolean;
begin
  DataR:= False;
  while ser.CanRead(1) do
  begin
    Memo1.Lines.Add(ser.Recvstring(1));
    DataR:= True;
  end;
  if not DataR then
    Memo1.Lines.Add('no data');
end;


procedure TfmMain.btInitClick(Sender: TObject);
begin
  if SendAndWait('AT') then
      if SendAndWait('AT+CMGF=1') then
      if  SendAndWait('AT+CSMP?') then
      if SendAndWait('at+cnmi=2,1') then
      if SendAndWait('AT+CSMP=17,167,0,0') then
    //  if SendAndWait('AT+CSMP=0,0,0,0') then
        Memo1.Lines.Add('Successfull Initialization');
end;

procedure TfmMain.btReadMsgsClick(Sender: TObject);
begin
  SendAndWait('at+cmgl="ALL"');
end;

procedure TfmMain.btExecClick(Sender: TObject);
begin
  SendAndWait(edCommand.Text);
end;

procedure TfmMain.btSendTextSMSClick(Sender: TObject);
begin
  // Set Text mode
  if SendAndWait('AT+CMGF=1') then
  // Prepair message
  if SendAndWait('AT+CMGS="' + edPhone.Text +'"', #13, 8) then
  begin
    // Send message text
    SendAndWait(edMsg.Text, #26, 7);

    // get sent message flag
    SendAndWait('at');
  end;
end;

procedure TfmMain.edMsgEnter(Sender: TObject);
begin
  if edMsg.Text = '<your text message>' then
    edMsg.Clear;

end;

procedure TfmMain.edPhoneEnter(Sender: TObject);
begin
  if edPhone.Text = '<phone num>' then
    edPhone.Clear;
end;

procedure TfmMain.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  ser.Free;
end;

procedure TfmMain.FormCreate(Sender: TObject);
begin
  ser:= TBlockSerial.Create;
end;

function TfmMain.SendAndWait(AText: string; Terminator: char = #13; WaitingSeconds: Integer = 3): boolean;
var
  Line: string;
  i: integer;
begin
  Memo1.Lines.Add(AText);
  ser.SendString(AText + Terminator);
  Result:= False;
  Line:= '';
  for i:= 1 to WaitingSeconds * 10 do
  begin
    Sleep(100);
    Application.ProcessMessages;
    while ser.CanRead(1) do
    begin
      Line:= ser.recvString(1);
      Memo1.Lines.Add(Line);
      if Trim(Line) <> '' then
        Result:= True;

      // Get the number of stored PDU message
      if Pos('+CMGW:', Trim(Line)) = 1 then
        PDUMsgNumber:= GetMsgNum(Line);

      Application.ProcessMessages;
    end;

    if Result then
      Break;
  end;
  Memo1.SelStart:= Length(Memo1.Text);
end;

function TfmMain.GetMsgNum(ALine: string): Integer;
begin
  Delete(ALine, 1, Pos(':', ALine));
  Result:= StrToInt(Trim(ALine));
end;

function TfmMain.SendPDU(MobileNumber, Msg: string): Boolean;
var
  ATC, Data: string;
begin
  StringToPDU(Msg, MobileNumber, edMsgCenter.Text, ATC, Data);
  { TODO : Read Message center from Configuration file }

  // Change to PDU mode
  SendAndWait('at+cmgf=0');

  // Store the message
  PDUMsgNumber:= -1;
  SendAndWait(ATC);

  SendAndWait(Data, #26);

  // wait stored message ID
  SendAndWait('AT');

  // Send stored message
  Result:= SendAndWait('at+cmss=' + IntToStr(PDUMsgNumber));

  // Delete stored message
  SendAndWait('at+cmgd=' + IntToStr(PDUMsgNumber));
end;

end.

